#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef _MSC_VER
//   Microsoft Visual C++ compiler still does not support C++ standard
#    pragma warning(disable: 4290)
#endif

#include "calc.h"

Calc* theCalc = 0;

Calc::Calc():
    actarray_capacity(1024),
    actarray_size(1024),
    actarray(new action[actarray_capacity]),
    stack(new RealStack())
{}

Calc::~Calc() {
    delete[] actarray;
    delete stack;
}

void Calc::addAction(const Calc::action& act) {
    if (actarray_size >= actarray_capacity) {
        if (actarray_capacity == 0)
            actarray_capacity = 1024;
        else
            actarray_capacity += actarray_capacity / 8;
        action* tmp = new action[actarray_capacity];
        memmove(tmp, actarray, actarray_capacity*sizeof(action));
        delete[] actarray;
        actarray = tmp;
    }
    actarray[actarray_size] = act;
    ++actarray_size;
}

const Calc::action& Calc::getAction(int i) const {
    return actarray[i];
}

void Calc::initialize() {
    actarray_size = 0;
    stack->init();
}

double Calc::calculate(double x) {
    stack->init();
    // printf("Calculate\n");

    for (int i = 0; i < actarray_size; i++) {
        double v1, v2;
        const action& act = getAction(i);
        switch (act.op) {
            case Double_const:
                // printf("const %lf, stack = %p\n", act.value, stack);
                stack->push(act.value);
                break;
            case Plus:
                // printf("Plus\n");
                v2 = stack->pop();
                v1 = stack->pop();
                stack->push(v1 + v2);
                break;
            case Minus:
                v2 = stack->pop();
                v1 = stack->pop();
                stack->push(v1 - v2);
                break;
            case Mult:
                v2 = stack->pop();
                v1 = stack->pop();
                stack->push(v1 * v2);
                break;
            case Div:
                v2 = stack->pop();
                v1 = stack->pop();
                stack->push(v1 / v2);
                break;
            case Uminus:
                v1 = stack->pop();
                stack->push(-v1);
                break;
            case X_value:
                stack->push(x);
                break;
        }
    }
    // printf("Top of stack\n");
    return stack->top();
}
