%{
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "scan.h"
#include "calc.h"
#include "parser.h"

int yylex(YYSTYPE *lvalp) { /* For pure parser */
    int ret = yyylex();

    /* printf("yyylex returned %d\n", ret); */

    *lvalp = yylval;
    return ret;
}

%}

%pure_parser

%token X LPARENT RPARENT DOUBLE_CONST ENDL ILLEGAL
%left PLUS MINUS
%left MULT DIV
%nonassoc UMINUS

%start f

%%
f : e ENDL
  | error ENDL  { }
  ;

e : e PLUS e    { theCalc->addAction(Calc::action(Calc::Plus)); }
  | e MINUS e   { theCalc->addAction(Calc::action(Calc::Minus)); }
  | e MULT e    { theCalc->addAction(Calc::action(Calc::Mult)); }
  | e DIV e     { theCalc->addAction(Calc::action(Calc::Div)); }
  | DOUBLE_CONST {
        theCalc->addAction(
            Calc::action(Calc::Double_const, $1.double_value)
        );
    }
  | X           { theCalc->addAction(Calc::action(Calc::X_value)); }
  | LPARENT e RPARENT
  | MINUS e %prec UMINUS { theCalc->addAction(Calc::action(Calc::Uminus)); }
  | PLUS e %prec UMINUS
  ;

%%

bool syntaxError = false;

#ifdef PARSER_TEST

int main() {
    char line[1024];
    double x;
    theCalc = new Calc();

    printf("Input the value of x:\n");
    // scanf("%lf\n", &x);
    gets(line);
    x = atof(line);

    while (true) {
        printf("Input an expression (including \"x\" and ariph. operations):\n");
        // scanf("%s", line);
        gets(line);
        if (line[0] == 0 || line[0] == 'q')
            break;

        // Initialize the scanner
        set_scanner_buffer(line, strlen(line));

        theCalc->initialize();
        syntaxError = false;

        // Call the parser
        yyparse();

        /*...
        // Print the reverse polish notation generated
        printf("Acr array:\n");
        for (int i = 0; i < theCalc->getActarraySize(); i++) {
            printf(
                "    %d %lf\n",
                theCalc->getAction(i).op,
                theCalc->getAction(i).value
            );
        }
        ...*/

        if (!syntaxError) {
            // Calculate the value of the expression, show the result
            double res = theCalc->calculate(x);
            printf("= %lf\n", res);
        }
    }
    delete theCalc;
    return 0;
}

#endif /* PARSER_TEST */

int yyerror(const char *s) {
    syntaxError = true;

#   ifdef PARSER_TEST
    printf("%s\n", s);
#   endif

    return 0;
}
