// DrawArea.cpp : implementation file
//

#include "stdafx.h"
#include <math.h>

#include "Graphic.h"
#include "DrawArea.h"
#include "GraphicDoc.h"
#include "GraphicView.h"

#pragma warning(disable: 4290)

#include "Calc/calc.h"

// CDrawArea

IMPLEMENT_DYNCREATE(CDrawArea, CView)

CDrawArea::CDrawArea():
    m_Font(),
    m_Brush(RGB(230, 230, 230)),
    m_GrayPen(PS_SOLID, 1, RGB(160, 160, 160)),
    m_BlackPen(PS_SOLID, 1, RGB(0, 0, 0)),
    m_RedPen(PS_SOLID, 1, RGB(255, 32, 32))
{
}

CDrawArea::~CDrawArea()
{
}

BEGIN_MESSAGE_MAP(CDrawArea, CView)
    ON_WM_ERASEBKGND()
    ON_WM_SIZE()
END_MESSAGE_MAP()


// CDrawArea drawing

void CDrawArea::OnDraw(CDC* pDC)
{
    // TODO: add draw code here
    /*...
    CRect clientRect;
    GetClientRect(&clientRect);
    pDC->FillRect(&clientRect, &m_Brush);
    ...*/
    DrawAxes(pDC);
    DrawGraph(pDC);
}

void CDrawArea::DrawAxes(CDC* pDC)
{
    CRect clientRect;
    GetClientRect(&clientRect);
    int x0 = 0;
    int x1 = clientRect.Width();
    int y0 = 0;
    int y1 = clientRect.Height();
    int ix, iy;

    CPen* oldPen = pDC->SelectObject(&m_GrayPen);

    CGraphicDoc* pDoc = GetDocument();

    // Draw grid
    double x = ceil(pDoc->m_xMin);
    while (x <= pDoc->m_xMax)
    {
        if (x != 0.)
        {
            ix = pDoc->MapX(x);
            pDC->MoveTo(ix, y0);
            pDC->LineTo(ix, y1);
        }
        x += 1.;
    }

    double y = ceil(pDoc->m_yMin);
    while (y <= pDoc->m_yMax)
    {
        if (y != 0.)
        {
            iy = pDoc->MapY(y);
            pDC->MoveTo(x0, iy);
            pDC->LineTo(x1, iy);
        }
        y += 1.;
    }

    // Draw axes
    pDC->SelectObject(&m_BlackPen);
    ix = pDoc->MapX(0.);
    pDC->MoveTo(ix, y0);
    pDC->LineTo(ix, y1);

    iy = pDoc->MapY(0.);
    pDC->MoveTo(x0, iy);
    pDC->LineTo(x1, iy);

    x0 = pDoc->MapX(-0.1);
    x1 = pDoc->MapX(0.1);
    iy = pDoc->MapY(1.);
    pDC->MoveTo(x0, iy);
    pDC->LineTo(x1, iy);

    y0 = pDoc->MapY(-0.1);
    y1 = pDoc->MapY(0.1);
    ix = pDoc->MapX(1.);
    pDC->MoveTo(ix, y0);
    pDC->LineTo(ix, y1);

    pDC->SelectObject(oldPen);
}

void CDrawArea::DrawGraph(CDC* pDC)
{
    CGraphicDoc* pDoc = GetDocument();
    CPen* oldPen = pDC->SelectObject(&m_RedPen);

    if (pDoc->m_bParseError)
    {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(RGB(255, 32, 32));
        pDC->DrawText(
            "Parse error",
            CRect(10, 10, 300, 80),
            DT_LEFT | DT_VCENTER
        );
    }
    else
    {
        int i;
        for (i = 0; i < pDoc->m_Curve.GetSize(); i++)
        {
            if (i == 0)
                pDC->MoveTo(pDoc->m_Curve[i]);
            else
                pDC->LineTo(pDoc->m_Curve[i]);
        }
    }
    pDC->SelectObject(oldPen);
}

// CDrawArea diagnostics

#ifdef _DEBUG
void CDrawArea::AssertValid() const
{
    CView::AssertValid();
}

void CDrawArea::Dump(CDumpContext& dc) const
{
    CView::Dump(dc);
}
#endif //_DEBUG

CGraphicDoc* CDrawArea::GetDocument() const {
    return (CGraphicDoc*) CView::GetDocument();
}

// CDrawArea message handlers

BOOL CDrawArea::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext)
{
    // TODO: Add your specialized code here and/or call the base class
    return CView::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
}

BOOL CDrawArea::OnEraseBkgnd(CDC* pDC)
{
    // TODO: Add your message handler code here and/or call default
    //... return CView::OnEraseBkgnd(pDC);
    CRect clientRect;
    GetClientRect(&clientRect);

    pDC->FillRect(&clientRect, &m_Brush);
    return TRUE;
}

void CDrawArea::OnInitialUpdate()
{
    CView::OnInitialUpdate();

    // TODO: Add your specialized code here and/or call the base class
    CGraphicDoc* pDoc = GetDocument();
    pDoc->m_pDrawArea = this;
    CalculateCoeff();
    pDoc->ComputeCurve();
}

void CDrawArea::CalculateCoeff() const
{
    CGraphicDoc* pDoc = GetDocument();
    CRect clientRect;
    GetClientRect(&clientRect);
    pDoc->m_DrawAreaWidth = clientRect.Width();
    if (pDoc->m_DrawAreaWidth == 0)
        pDoc->m_DrawAreaWidth = 1;
    pDoc->m_DrawAreaHeight = clientRect.Height();
    if (pDoc->m_DrawAreaHeight == 0)
        pDoc->m_DrawAreaHeight = 1;

    // Tune ymin, ymax to preserve the aspect ratio
    double dx = pDoc->m_xMax - pDoc->m_xMin;
    double dy = pDoc->m_yMax - pDoc->m_yMin;
    double dy_corrected = dx *
        (double) pDoc->m_DrawAreaHeight /
        (double) pDoc->m_DrawAreaWidth;
    double ycenter = (pDoc->m_yMin + pDoc->m_yMax) / 2.;
    double ycoeff = dy_corrected / dy;
    pDoc->m_yMax = ycenter + (pDoc->m_yMax - ycenter) * ycoeff;
    pDoc->m_yMin = ycenter + (pDoc->m_yMin - ycenter) * ycoeff;

    pDoc->m_CoeffX = (double) pDoc->m_DrawAreaWidth /
        (pDoc->m_xMax - pDoc->m_xMin);
    pDoc->m_CoeffY = (double) pDoc->m_DrawAreaHeight /
        (pDoc->m_yMax - pDoc->m_yMin);

    pDoc->m_pView->ShowXYMinMax();
}

void CDrawArea::OnSize(UINT nType, int cx, int cy)
{
    CView::OnSize(nType, cx, cy);

    // TODO: Add your message handler code here
    CGraphicDoc* pDoc = GetDocument();
    if (theCalc != 0)
    {
        CalculateCoeff();
        pDoc->ComputeCurve();
    }
}
