// GraphicDoc.cpp : implementation of the CGraphicDoc class
//

#include "stdafx.h"
#include <math.h>

#include "Graphic.h"
#include "GraphicDoc.h"

#pragma warning(disable: 4290)

// Parser definitions
#include "Calc/scan.h"
#include "Calc/parser.h"
#include "Calc/calc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CGraphicDoc

IMPLEMENT_DYNCREATE(CGraphicDoc, CDocument)

BEGIN_MESSAGE_MAP(CGraphicDoc, CDocument)
END_MESSAGE_MAP()


// CGraphicDoc construction/destruction

CGraphicDoc::CGraphicDoc():
    m_pView(NULL),
    m_pDrawArea(NULL),
    m_Curve(),
    m_xMin(-12.),
    m_xMax(12.),
    m_yMin(-9.),
    m_yMax(9),
    m_DrawAreaWidth(400),
    m_DrawAreaHeight(300),
    m_CoeffX((double) m_DrawAreaWidth / (m_xMax-m_xMin)),
    m_CoeffY((double) m_DrawAreaHeight / (m_yMax-m_yMin)),

    m_sFunction("4 / (1 + 0.25*x*x)"),
    m_bParseError(FALSE)
{
    // TODO: add one-time construction code here

}

CGraphicDoc::~CGraphicDoc()
{
    delete theCalc;
    theCalc = 0;
}

BOOL CGraphicDoc::OnNewDocument()
{
    if (!CDocument::OnNewDocument())
        return FALSE;

    // TODO: add reinitialization code here
    // (SDI documents will reuse this document)

    return TRUE;
}

// CGraphicDoc serialization

void CGraphicDoc::Serialize(CArchive& ar)
{
    if (ar.IsStoring())
    {
        // TODO: add storing code here
    }
    else
    {
        // TODO: add loading code here
    }
}

// CGraphicDoc diagnostics

#ifdef _DEBUG
void CGraphicDoc::AssertValid() const
{
    CDocument::AssertValid();
}

void CGraphicDoc::Dump(CDumpContext& dc) const
{
    CDocument::Dump(dc);
}
#endif //_DEBUG

// CGraphicDoc commands

static const int GRAPH_STEP = 2;    // Means 2 pixels

void CGraphicDoc::ComputeCurve()
{
    m_Curve.RemoveAll();
    if (m_bParseError)
        return;
    double dx = (double) GRAPH_STEP / m_CoeffX;  // Step = 2 pixels
    double x = m_xMin;
    while (x <= m_xMax)
    {
        double y = Function(x);
        m_Curve.Add(CPoint(MapX(x), MapY(y)));
        x += dx;
    }
}

int CGraphicDoc::MapX(double x) const
{
    return (int) ((x - m_xMin) * m_CoeffX + 0.5);
}

int CGraphicDoc::MapY(double y) const
{
    return (int) ((m_yMax - y) * m_CoeffY + 0.5);
}

double CGraphicDoc::Function(double x) const
{
    if (m_bParseError)
        return 0.;
    return theCalc->calculate(x);
}

BOOL CGraphicDoc::ParseFunction(LPCTSTR func)
{
    if (theCalc == 0)
        theCalc = new Calc();

    syntaxError = false;
    m_sFunction = func;
    set_scanner_buffer(func, (int) strlen(func));
    theCalc->initialize();

    yyparse();
    m_bParseError = (BOOL) syntaxError;

    return !m_bParseError;
}
