// GraphicView.cpp : implementation of the CGraphicView class
//

#include "stdafx.h"
#include "Graphic.h"

#include "GraphicDoc.h"
#include "GraphicView.h"
#include "DrawArea.h"

#pragma warning(disable: 4290)

#include "Calc/Calc.h"
#include "Calc/scan.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CGraphicView

IMPLEMENT_DYNCREATE(CGraphicView, CFormView)

BEGIN_MESSAGE_MAP(CGraphicView, CFormView)
    ON_BN_CLICKED(IDC_DRAW, OnBnClickedDraw)
    ON_EN_CHANGE(IDC_XMIN, OnEnChangeXmin)
    ON_EN_KILLFOCUS(IDC_XMIN, OnEnKillfocusXmin)
    ON_EN_CHANGE(IDC_XMAX, OnEnChangeXmax)
    ON_EN_KILLFOCUS(IDC_XMAX, OnEnKillfocusXmax)
    ON_EN_CHANGE(IDC_YMIN, OnEnChangeYmin)
    ON_EN_KILLFOCUS(IDC_YMIN, OnEnKillfocusYmin)
    ON_EN_CHANGE(IDC_YMAX, OnEnChangeYmax)
    ON_EN_KILLFOCUS(IDC_YMAX, OnEnKillfocusYmax)
    ON_EN_CHANGE(IDC_FUNC, OnEnChangeFunc)
    ON_EN_KILLFOCUS(IDC_FUNC, OnEnKillfocusFunc)
    ON_WM_SIZE()
END_MESSAGE_MAP()

// CGraphicView construction/destruction

CGraphicView::CGraphicView()
    : CFormView(CGraphicView::IDD)

    , m_bXMinChanged(FALSE)
    , m_bXMaxChanged(FALSE)
    , m_bYMinChanged(FALSE)
    , m_bYMaxChanged(FALSE)
    , m_bFuncChanged(FALSE)

    , m_OldSizeX(0)
    , m_OldSizeY(0)

    , m_sXMax(_T(""))
    , m_sXMin(_T(""))
    , m_sYMin(_T(""))
    , m_sYMax(_T(""))
    , m_sFunc(_T(""))
{
    // TODO: add construction code here
}

CGraphicView::~CGraphicView()
{
}

void CGraphicView::DoDataExchange(CDataExchange* pDX)
{
    CFormView::DoDataExchange(pDX);
    DDX_Text(pDX, IDC_XMAX, m_sXMax);
    DDX_Text(pDX, IDC_XMIN, m_sXMin);
    DDX_Text(pDX, IDC_YMIN, m_sYMin);
    DDX_Text(pDX, IDC_YMAX, m_sYMax);
    DDX_Text(pDX, IDC_FUNC, m_sFunc);
}

BOOL CGraphicView::PreCreateWindow(CREATESTRUCT& cs)
{
    // TODO: Modify the Window class or styles here by modifying
    //  the CREATESTRUCT cs

    return CFormView::PreCreateWindow(cs);
}

void CGraphicView::OnInitialUpdate()
{
    CGraphicDoc* pDoc = GetDocument();

    CFormView::OnInitialUpdate();
    GetParentFrame()->RecalcLayout();
    ResizeParentToFit();

    pDoc->ParseFunction(pDoc->m_sFunction);

    m_sFunc = pDoc->m_sFunction;
    ShowXYMinMax();

    CRect r;
    GetClientRect(&r);
    m_OldSizeX = r.Width();
    m_OldSizeY = r.Height();
}

void CGraphicView::ShowXYMinMax()
{
    CGraphicDoc* pDoc = GetDocument();

    m_sXMin.Format("%.2lf", pDoc->m_xMin);
    m_sXMax.Format("%.2lf", pDoc->m_xMax);
    m_sYMin.Format("%.2lf", pDoc->m_yMin);
    m_sYMax.Format("%.2lf", pDoc->m_yMax);
    UpdateData(FALSE);
}

// CGraphicView diagnostics

#ifdef _DEBUG
void CGraphicView::AssertValid() const
{
    CFormView::AssertValid();
}

void CGraphicView::Dump(CDumpContext& dc) const
{
    CFormView::Dump(dc);
}

CGraphicDoc* CGraphicView::GetDocument() const // non-debug version is inline
{
    ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGraphicDoc)));
    return (CGraphicDoc*)m_pDocument;
}
#endif //_DEBUG


// CGraphicView message handlers

void CGraphicView::OnBnClickedDraw()
{
    // TODO: Add your control notification handler code here
}

BOOL CGraphicView::Create(
    LPCTSTR lpszClassName, LPCTSTR lpszWindowName,
    DWORD dwStyle, const RECT& rect, CWnd* pParentWnd,
    UINT nID, CCreateContext* pContext
)
{
    // TODO: Add your specialized code here and/or call the base class
    if (!CFormView::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext))
        return FALSE;

    CWnd* w = GetDlgItem(IDC_DRAW_AREA);
    if (w == NULL)
        return FALSE;
    CRect r;
    w->GetWindowRect(&r);
    ScreenToClient(&r);

    CDrawArea* drawArea = (CDrawArea*) 
        RUNTIME_CLASS(CDrawArea)->CreateObject();

    if (!drawArea->Create(
        NULL, NULL, AFX_WS_DEFAULT_VIEW,
        r, this, UINT(-1), pContext
    ))
        return FALSE;

    CGraphicDoc* pDoc = GetDocument();
    pDoc->m_pView = this;

    return TRUE;
}

void CGraphicView::OnEnChangeXmin()
{
    // TODO:  If this is a RICHEDIT control, the control will not
    // send this notification unless you override the CFormView::OnInitDialog()
    // function and call CRichEditCtrl().SetEventMask()
    // with the ENM_CHANGE flag ORed into the mask.

    // TODO:  Add your control notification handler code here
    m_bXMinChanged = TRUE;
}

void CGraphicView::OnEnKillfocusXmin()
{
    // TODO: Add your control notification handler code here
    if (m_bXMinChanged)
    {
        CGraphicDoc* pDoc = GetDocument();
        UpdateData();
        pDoc->m_xMin = atof(m_sXMin);
        pDoc->m_pDrawArea->CalculateCoeff();
        pDoc->ComputeCurve();
        pDoc->m_pDrawArea->Invalidate(TRUE);
        m_bXMinChanged = FALSE;
    }
}

void CGraphicView::OnEnChangeXmax()
{
    // TODO:  If this is a RICHEDIT control, the control will not
    // send this notification unless you override the CFormView::OnInitDialog()
    // function and call CRichEditCtrl().SetEventMask()
    // with the ENM_CHANGE flag ORed into the mask.

    // TODO:  Add your control notification handler code here
    m_bXMaxChanged = TRUE;
}

void CGraphicView::OnEnKillfocusXmax()
{
    // TODO: Add your control notification handler code here
    if (m_bXMaxChanged)
    {
        CGraphicDoc* pDoc = GetDocument();
        UpdateData();
        pDoc->m_xMax = atof(m_sXMax);
        pDoc->m_pDrawArea->CalculateCoeff();
        pDoc->ComputeCurve();
        pDoc->m_pDrawArea->Invalidate(TRUE);
        m_bXMaxChanged = FALSE;
    }
}

void CGraphicView::OnEnChangeYmin()
{
    // TODO:  If this is a RICHEDIT control, the control will not
    // send this notification unless you override the CFormView::OnInitDialog()
    // function and call CRichEditCtrl().SetEventMask()
    // with the ENM_CHANGE flag ORed into the mask.

    // TODO:  Add your control notification handler code here
    m_bYMinChanged = TRUE;
}

void CGraphicView::OnEnKillfocusYmin()
{
    // TODO: Add your control notification handler code here
    if (m_bYMinChanged)
    {
        CGraphicDoc* pDoc = GetDocument();
        UpdateData();
        double dy = pDoc->m_yMax - pDoc->m_yMin;
        pDoc->m_yMin = atof(m_sYMin);
        pDoc->m_yMax = pDoc->m_yMin + dy;
        pDoc->m_pDrawArea->CalculateCoeff();
        pDoc->ComputeCurve();
        pDoc->m_pDrawArea->Invalidate(TRUE);
        m_bYMinChanged = FALSE;
    }
}

void CGraphicView::OnEnChangeYmax()
{
    // TODO:  If this is a RICHEDIT control, the control will not
    // send this notification unless you override the CFormView::OnInitDialog()
    // function and call CRichEditCtrl().SetEventMask()
    // with the ENM_CHANGE flag ORed into the mask.

    // TODO:  Add your control notification handler code here
    m_bYMaxChanged = TRUE;
}

void CGraphicView::OnEnKillfocusYmax()
{
    // TODO: Add your control notification handler code here
    if (m_bYMaxChanged)
    {
        CGraphicDoc* pDoc = GetDocument();
        UpdateData();
        double dy = pDoc->m_yMax - pDoc->m_yMin;
        pDoc->m_yMax = atof(m_sYMax);
        pDoc->m_yMin = pDoc->m_yMax - dy;
        pDoc->m_pDrawArea->CalculateCoeff();
        pDoc->ComputeCurve();
        pDoc->m_pDrawArea->Invalidate(TRUE);
        m_bYMinChanged = FALSE;
    }
}

void CGraphicView::OnEnChangeFunc()
{
    // TODO:  If this is a RICHEDIT control, the control will not
    // send this notification unless you override the CFormView::OnInitDialog()
    // function and call CRichEditCtrl().SetEventMask()
    // with the ENM_CHANGE flag ORed into the mask.

    // TODO:  Add your control notification handler code here
    m_bFuncChanged = TRUE;
}

void CGraphicView::OnEnKillfocusFunc()
{
    // TODO: Add your control notification handler code here
    if (m_bFuncChanged)
    {
        CGraphicDoc* pDoc = GetDocument();
        UpdateData();

        if (pDoc->ParseFunction(m_sFunc))
            pDoc->ComputeCurve();
        else
            pDoc->m_Curve.RemoveAll();
        pDoc->m_pDrawArea->Invalidate(TRUE);
        m_bFuncChanged = FALSE;
    }
}

void CGraphicView::OnSize(UINT nType, int cx, int cy)
{
    CFormView::OnSize(nType, cx, cy);

    // TODO: Add your message handler code here
    CGraphicDoc* pDoc = GetDocument();
    int dx = cx - m_OldSizeX;
    int dy = cy - m_OldSizeY;
        
    if (pDoc->m_pDrawArea != 0)
    {
        CRect r;
        pDoc->m_pDrawArea->GetWindowRect(&r);
        pDoc->m_pDrawArea->SetWindowPos(
            NULL,
            0, 0,
            r.Width() + dx,
            r.Height() + dy,
            SWP_NOMOVE | SWP_NOZORDER
        );
    }

    m_OldSizeX = cx;
    m_OldSizeY = cy;
}
